@@ -641,6 +641,72 @@ class LoadTest:
                     {"clients": 96, "requests": 1200000},
                 ],
             },
+            "1k-1k-worker": {
+                "title": "1k-1k files, varying worker, {protocol} ({measure})",
+                "class": UrlsLoadTest,
+                "location": "/",
+                "file_count": 100,
+                "file_sizes": [1],
+                "start_servers": 4,
+                "h2_workers_min": 1,
+                "h2_workers_max": 40,
+                "requests": 50000,
+                "warmup": False,
+                "measure": "req/s",
+                "protocol": 'h2',
+                "max_parallel": 50,
+                "row0_title": "max requests",
+                "row_title": "{clients}c",
+                "rows": [
+                    {"clients": 48, "requests": 500000},
+                ],
+                "col_title": "{h2_workers_min}-{h2_workers_max}",
+                "clients": 1,
+                "columns": [
+                    {"h2_workers_min": 1, "h2_workers_max": 25, },
+                    {"h2_workers_min": 1, "h2_workers_max": 50, },
+                    {"h2_workers_min": 1, "h2_workers_max": 100, },
+                    {"h2_workers_min": 1, "h2_workers_max": 250, },
+                    {"h2_workers_min": 1, "h2_workers_max": 500, },
+                    {"h2_workers_min": 1, "h2_workers_max": 750, },
+                    {"h2_workers_min": 1, "h2_workers_max": 1000, },
+                    {"h2_workers_min": 1, "h2_workers_max": 5000, },
+                    {"h2_workers_min": 1, "h2_workers_max": 10000, },
+                ],
+            },
+            "1k-1k-worker-max": {
+                "title": "1k-1k files, varying worker min==max, {protocol} ({measure})",
+                "class": UrlsLoadTest,
+                "location": "/",
+                "file_count": 100,
+                "file_sizes": [1],
+                "start_servers": 4,
+                "h2_workers_min": 1,
+                "h2_workers_max": 40,
+                "requests": 50000,
+                "warmup": False,
+                "measure": "req/s",
+                "protocol": 'h2',
+                "max_parallel": 50,
+                "row0_title": "max requests",
+                "row_title": "{clients}c",
+                "rows": [
+                    {"clients": 48, "requests": 500000},
+                ],
+                "col_title": "{h2_workers_min}-{h2_workers_max}",
+                "clients": 1,
+                "columns": [
+                    {"h2_workers_min": 25, "h2_workers_max": 25, },
+                    {"h2_workers_min": 50, "h2_workers_max": 50, },
+                    {"h2_workers_min": 100, "h2_workers_max": 100, },
+                    {"h2_workers_min": 250, "h2_workers_max": 250, },
+                    {"h2_workers_min": 500, "h2_workers_max": 500, },
+                    {"h2_workers_min": 750, "h2_workers_max": 750, },
+                    {"h2_workers_min": 1000, "h2_workers_max": 1000, },
+                    {"h2_workers_min": 5000, "h2_workers_max": 5000, },
+                    {"h2_workers_min": 10000, "h2_workers_max": 10000, },
+                ],
+            },
             "1p-10c": {
                 "title": "fast workers(fw), {start_servers} process, {clients} conn, {max_parallel} parallel, 1m req, ({measure})",
                 "class": UrlsLoadTest,
